/*
 * Copyright 2023 dorkbox, llc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dorkbox.network.handshake

import org.agrona.collections.Object2IntHashMap
import java.net.InetAddress

/**
 *
 */
internal class ConnectionCounts {
    private val connectionsPerIpCounts = Object2IntHashMap<InetAddress>(-1)

    @Synchronized
    fun get(inetAddress: InetAddress): Int {
        return connectionsPerIpCounts.getOrPut(inetAddress) { 0 }
    }

    @Synchronized
    fun increment(inetAddress: InetAddress, currentCount: Int) {
        connectionsPerIpCounts[inetAddress] = currentCount + 1
    }

    @Synchronized
    fun decrement(inetAddress: InetAddress, currentCount: Int) {
        connectionsPerIpCounts[inetAddress] = currentCount - 1
    }

    @Synchronized
    fun decrementSlow(inetAddress: InetAddress) {
        if (connectionsPerIpCounts.containsKey(inetAddress)) {
            val defaultVal = connectionsPerIpCounts.getValue(inetAddress)
            connectionsPerIpCounts[inetAddress] = defaultVal - 1
        }
    }

    @Synchronized
    fun isEmpty(): Boolean {
        return connectionsPerIpCounts.isEmpty()
    }

    @Synchronized
    override fun toString(): String {
        return connectionsPerIpCounts.entries.map { it.key }.joinToString()
    }
}
